-- {COPYRIGHT-TOP} ------------------------------------------------
-- IBM Confidential
-- OCO Source Materials
-- 5724-V51
--
--  Copyright IBM Corp. 2000, 2008 All Rights Reserved.
--
-- The source code for this program is not published or otherwise
-- divested of its trade secrets, irrespective of what has been
-- deposited with the U.S. Copyright Office.
-- {COPYRIGHT-END} ------------------------------------------------


-- DB2 UDB UDF 
--
-- Author    Beshwa
--
-- Usage of UDF: 
-- SSINSTR (CHAR1, CHAR2, N, M)
-- or
-- SSINSTR (expression1, expression2, startfrom)
-- or
-- SSINSTR (expression1, expression2)
--
-- Synopsis : Return the position of Expression1 in Expression2
--               Duplication of functionality as provided by Oracle INSTR function
--            

-- Cleanup
DROP FUNCTION SSINSTR (VARCHAR(1000), VARCHAR(1000));
DROP FUNCTION SSINSTR (VARCHAR(1000), VARCHAR(1000), INTEGER);
DROP FUNCTION SSINSTR (VARCHAR(1000), VARCHAR(1000), INTEGER, INTEGER);

CREATE FUNCTION SSINSTR (CHAR1 VarChar(1000), CHAR2 VarChar(1000), N integer, M integer)
 RETURNS Integer
 SPECIFIC SSINSTRBaseUDF
 LANGUAGE SQL
 CONTAINS SQL
 NO EXTERNAL ACTION
 DETERMINISTIC
 RETURN 
WITH
 Repeat1 (Pos, R) AS
(Values (N-1, 0)
Union All
Select locate(CHAR2,CHAR1,Pos+1), R+1
  From Repeat1
 Where Pos < length(CHAR1)
   and R < M
   and N > 0
)

,Repeat2 (Pos, R) AS
(Values (length(CHAR1)+N, 0)
Union All
Select
 Pos-1
,CASE WHEN substr(CHAR1,Pos,length(CHAR2)) = CHAR2 THEN R+1 ELSE R END
  From Repeat2
 Where Pos > 0
   and R < M
   and N < 0
)

SELECT CASE
       WHEN N > 0 THEN Rp1.Pos*(1-sign(M-Rp1.R))
       WHEN N < 0 THEN (Rp2.Pos + 1)*(1-sign(M-Rp2.R))
       ELSE 0
       END
  FROM Repeat1 Rp1
      ,Repeat2 Rp2
 WHERE
       Rp1.Pos = (Select max(Pos) From Repeat1)
   AND Rp2.Pos = (Select min(Pos) From Repeat2)
;
 



-- Overloaded for situation when order parameter is not submitted
-- Bhavani
-- Changed by Bhavani on 10-Nov-2002 to resolve a bug ( when length of expression1 
-- is 1 )
CREATE FUNCTION SSINSTR (expression1 VARCHAR(1000), expression2 VARCHAR(1000), startfrom INTEGER)
RETURNS INTEGER
SPECIFIC SSINSTRBOverloadP3
LANGUAGE SQL
CONTAINS SQL
NO EXTERNAL ACTION
DETERMINISTIC
RETURN 
CASE 
WHEN LENGTH(expression1)= 1 THEN SSINSTR(expression1, expression2, startfrom, 0)
WHEN LENGTH(expression2)= 1 THEN SSINSTR(expression1, expression2, startfrom, 0)
ELSE SSINSTR(expression1, expression2, startfrom, 1)
END;




-- Overloaded for situation when startfrom and order are not submitted

CREATE FUNCTION SSINSTR (expression1 VARCHAR(1000), expression2 VARCHAR(1000))
RETURNS INTEGER
SPECIFIC SSINSTRBOverloadP2
LANGUAGE SQL
CONTAINS SQL
NO EXTERNAL ACTION
DETERMINISTIC
RETURN 
LOCATE(expression1, expression1, 1);

commit;

terminate;
 
